unit FormOptions;

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  Spin, ComCtrls, ExtCtrls, Target, Theme;

type

  { TFormOptions }

  TFormOptions = class(TForm)
    ButtonOk: TButton;
    ButtonCancel: TButton;
    CheckBoxReopenLastFile: TCheckBox;
    CheckBoxDPIAuto: TCheckBox;
    CheckBoxOptimizeAddSub: TCheckBox;
    CheckBoxDebugEnabled: TCheckBox;
    CheckBoxOptimizeCopyMultiply: TCheckBox;
    CheckBoxOptimizeSetZero: TCheckBox;
    CheckBoxOptimizeMerge: TCheckBox;
    CheckBoxOptimizeRelativeIndexes: TCheckBox;
    ComboBoxLanguage: TComboBox;
    ComboBoxOptimization: TComboBox;
    ComboBoxTheme: TComboBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    LabelDPI: TLabel;
    LabelTheme: TLabel;
    LabelX: TLabel;
    PageControl1: TPageControl;
    Panel1: TPanel;
    Panel2: TPanel;
    SpinEditCellSize: TSpinEdit;
    SpinEditDPIX: TSpinEdit;
    SpinEditDPIY: TSpinEdit;
    SpinEditMemorySize: TSpinEdit;
    TabSheetGeneral: TTabSheet;
    TabSheetBuild: TTabSheet;
    procedure CheckBoxDPIAutoChange(Sender: TObject);
    procedure CheckBoxOptimizeCopyMultiplyChange(Sender: TObject);
    procedure CheckBoxOptimizeRelativeIndexesChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  public
    procedure UpdateInterface;
    procedure Load;
    procedure Save;
  end;


implementation

{$R *.lfm}

uses
  Core, Languages;

{ TFormOptions }

procedure TFormOptions.FormShow(Sender: TObject);
begin
  Core.Core.PersistentForm1.Load(Self);
end;

procedure TFormOptions.UpdateInterface;
begin
  SpinEditDPIX.Enabled := not CheckBoxDPIAuto.Checked;
  SpinEditDPIY.Enabled := not CheckBoxDPIAuto.Checked;
  LabelDPI.Enabled := not CheckBoxDPIAuto.Checked;
  LabelX.Enabled := not CheckBoxDPIAuto.Checked;
  if CheckBoxDPIAuto.Checked then begin
    SpinEditDPIX.Value := ScreenInfo.PixelsPerInchX;
    SpinEditDPIY.Value := ScreenInfo.PixelsPerInchY;
  end;
  CheckBoxOptimizeAddSub.Enabled := ComboBoxOptimization.ItemIndex > 0;
  CheckBoxOptimizeSetZero.Enabled := ComboBoxOptimization.ItemIndex > 0;
  CheckBoxOptimizeMerge.Enabled := ComboBoxOptimization.ItemIndex > 0;
  CheckBoxOptimizeRelativeIndexes.Enabled := ComboBoxOptimization.ItemIndex > 0;
  CheckBoxOptimizeCopyMultiply.Enabled := ComboBoxOptimization.ItemIndex > 0;
end;

procedure TFormOptions.Load;
begin
  with Core.Core do begin
    ComboBoxLanguage.ItemIndex := ComboBoxLanguage.Items.IndexOfObject(Translator.Language);
    if ComboBoxLanguage.ItemIndex = -1 then ComboBoxLanguage.ItemIndex := 0;
    ComboBoxTheme.ItemIndex := ComboBoxTheme.Items.IndexOfObject(ThemeManager.Theme);
    if ComboBoxTheme.ItemIndex = -1 then ComboBoxTheme.ItemIndex := 0;
    CheckBoxReopenLastFile.Checked := OpenProjectOnStart;
    CheckBoxOptimizeAddSub.Checked := Optimizations.AddSub;
    CheckBoxOptimizeSetZero.Checked := Optimizations.SetZero;
    CheckBoxOptimizeMerge.Checked := Optimizations.Merge;
    CheckBoxOptimizeRelativeIndexes.Checked := Optimizations.RelativeIndexes;
    CheckBoxOptimizeCopyMultiply.Checked := Optimizations.CopyMultiply;
    SpinEditDPIX.Value := ScaleDPI1.DPI.X;
    SpinEditDPIY.Value := ScaleDPI1.DPI.Y;
    CheckBoxDPIAuto.Checked := ScaleDPI1.AutoDetect;
    ComboBoxOptimization.ItemIndex := Integer(OptimizationLevel);
    SpinEditCellSize.Value := CellSize;
    SpinEditMemorySize.Value := MemorySize;
    CheckBoxDebugEnabled.Checked := DebugEnabled;
    if OptionsFormTabIndex < PageControl1.PageCount then
      PageControl1.TabIndex := OptionsFormTabIndex
      else PageControl1.TabIndex := 0;
  end;
  UpdateInterface;
end;

procedure TFormOptions.Save;
begin
  with Core.Core do begin
  if ComboBoxLanguage.ItemIndex <> -1 then
    Translator.Language := TLanguage(ComboBoxLanguage.Items.Objects[ComboBoxLanguage.ItemIndex]);
  if ComboBoxTheme.ItemIndex <> -1 then
    ThemeManager.Theme := TTheme(ComboBoxTheme.Items.Objects[ComboBoxTheme.ItemIndex]);
    OpenProjectOnStart := CheckBoxReopenLastFile.Checked;
    ScaleDPI1.DPI := Point(SpinEditDPIX.Value, SpinEditDPIY.Value);
    ScaleDPI1.AutoDetect := CheckBoxDPIAuto.Checked;
    OptimizationLevel := TCompilerOptimization(ComboBoxOptimization.ItemIndex);
    CellSize := SpinEditCellSize.Value;
    MemorySize := SpinEditMemorySize.Value;
    Optimizations.AddSub := CheckBoxOptimizeAddSub.Checked;
    Optimizations.SetZero := CheckBoxOptimizeSetZero.Checked;
    Optimizations.Merge := CheckBoxOptimizeMerge.Checked;
    Optimizations.RelativeIndexes := CheckBoxOptimizeRelativeIndexes.Checked;
    Optimizations.CopyMultiply := CheckBoxOptimizeCopyMultiply.Checked;
    DebugEnabled := CheckBoxDebugEnabled.Checked;
    OptionsFormTabIndex := PageControl1.TabIndex;
  end;
end;

procedure TFormOptions.CheckBoxDPIAutoChange(Sender: TObject);
begin
  UpdateInterface;
end;

procedure TFormOptions.CheckBoxOptimizeCopyMultiplyChange(Sender: TObject);
begin
  // Needed for optimize copy multiply
  if CheckBoxOptimizeCopyMultiply.Checked then
    CheckBoxOptimizeRelativeIndexes.Checked := True;
end;

procedure TFormOptions.CheckBoxOptimizeRelativeIndexesChange(Sender: TObject);
begin
  // Needed for optimize copy multiply
  if not CheckBoxOptimizeRelativeIndexes.Checked then
    CheckBoxOptimizeCopyMultiply.Checked := False;
end;

procedure TFormOptions.FormClose(Sender: TObject; var CloseAction: TCloseAction
  );
begin
  Core.Core.PersistentForm1.Save(Self);
end;

procedure TFormOptions.FormCreate(Sender: TObject);
begin
  with Core.Core do begin
    Translator.TranslateComponentRecursive(Self);
    ThemeManager.UseTheme(Self);
    ThemeManager.Themes.LoadToStrings(ComboBoxTheme.Items);
    Translator.LanguageListToStrings(ComboBoxLanguage.Items);
  end;
  ComboBoxOptimization.Items.Clear;
  ComboBoxOptimization.Items.Add(CompilerOptimizationText[coNone]);
  ComboBoxOptimization.Items.Add(CompilerOptimizationText[coNormal]);
end;

end.

