unit FormSourceCode;

interface

uses
  Classes, SysUtils, FileUtil, SynEdit, SynHighlighterIni, Forms, Controls,
  Graphics, Dialogs, Menus, ActnList,
  SynEditHighlighter, SynHighlighterAny, SynHighlighterJava, BFHighlighter;

type

  { TFormSourceCode }

  TFormSourceCode = class(TForm)
    AGenerateText: TAction;
    ACutToClipboard: TAction;
    ASelectAll: TAction;
    APasteFromClipboard: TAction;
    ACopyToClipboard: TAction;
    AClear: TAction;
    AFormatSource: TAction;
    AShrinkSource: TAction;
    AGenerateNumber: TAction;
    ActionList1: TActionList;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem21: TMenuItem;
    MenuItem23: TMenuItem;
    MenuItem26: TMenuItem;
    MenuItem29: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem36: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    Separator1: TMenuItem;
    PopupMenuSource: TPopupMenu;
    SynEditSource: TSynEdit;
    SynJavaSyn1: TSynJavaSyn;
    procedure AClearExecute(Sender: TObject);
    procedure ACopyToClipboardExecute(Sender: TObject);
    procedure ACutToClipboardExecute(Sender: TObject);
    procedure AFormatSourceExecute(Sender: TObject);
    procedure AGenerateNumberExecute(Sender: TObject);
    procedure AGenerateTextExecute(Sender: TObject);
    procedure APasteFromClipboardExecute(Sender: TObject);
    procedure ASelectAllExecute(Sender: TObject);
    procedure AShrinkSourceExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MemoSourceChange(Sender: TObject);
    procedure MemoSourceKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure MemoSourceKeyPress(Sender: TObject; var Key: char);
    procedure MemoSourceKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState
      );
    procedure MemoSourceMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    BFHighlighter: TSynBrainFuckHl;
    FOnUpdateInterface: TNotifyEvent;
    FOnUpdateStatusBar: TNotifyEvent;
  public
    procedure UpdateInterface;
    procedure UpdateTheme;
    property OnUpdateStatusBar: TNotifyEvent read FOnUpdateStatusBar
      write FOnUpdateStatusBar;
    property OnUpdateInterface: TNotifyEvent read FOnUpdateInterface
      write FOnUpdateInterface;
  end;


implementation

{$R *.lfm}

uses
  Core, BFCodeTools;

resourcestring
  SNumberGeneration = 'Number generation';
  SEnterNumber = 'Enter number';
  STextGeneration = 'Text generation';
  SEnterText = 'Enter text';

{ TFormSourceCode }

procedure TFormSourceCode.AGenerateNumberExecute(Sender: TObject);
var
  Number: Integer;
  NumberText: string;
begin
  NumberText := InputBox(SNumberGeneration, SEnterNumber, '1');
  if TryStrToInt(NumberText, Number) then begin
    SynEditSource.SelText := GenerateNumber(Number);
  end;
end;

procedure TFormSourceCode.AGenerateTextExecute(Sender: TObject);
var
  Text: string;
begin
  Text := InputBox(STextGeneration, SEnterText, '');
  SynEditSource.SelText := GenerateText(Text);
end;

procedure TFormSourceCode.APasteFromClipboardExecute(Sender: TObject);
begin
  SynEditSource.PasteFromClipboard;
end;

procedure TFormSourceCode.ASelectAllExecute(Sender: TObject);
begin
  SynEditSource.SelectAll;
end;

procedure TFormSourceCode.AFormatSourceExecute(Sender: TObject);
begin
  SynEditSource.Text := FormatCode(SynEditSource.Text);
  MemoSourceChange(nil);
end;

procedure TFormSourceCode.ACopyToClipboardExecute(Sender: TObject);
begin
  SynEditSource.CopyToClipboard;
end;

procedure TFormSourceCode.ACutToClipboardExecute(Sender: TObject);
begin
  SynEditSource.CutToClipboard;
end;

procedure TFormSourceCode.AClearExecute(Sender: TObject);
begin
  SynEditSource.Clear;
end;

procedure TFormSourceCode.AShrinkSourceExecute(Sender: TObject);
begin
  SynEditSource.Text := ShrinkCode(SynEditSource.Text);
  MemoSourceChange(nil);
end;

procedure TFormSourceCode.FormCreate(Sender: TObject);
begin
  Core.Core.Translator.TranslateComponentRecursive(Self);
  Core.Core.ThemeManager.UseTheme(Self);

  BFHighlighter := TSynBrainFuckHl.Create(nil);
  SynEditSource.Highlighter := BFHighlighter;
end;

procedure TFormSourceCode.FormDestroy(Sender: TObject);
begin
  SynEditSource.Highlighter := nil;
  FreeAndNil(BFHighlighter);
end;

procedure TFormSourceCode.UpdateInterface;
begin
end;

procedure TFormSourceCode.UpdateTheme;
const
  clLightBlue = TColor($FF8080);
  clLightGreen = TColor($80FF80);
  clLightRed = TColor($8080FF);
begin
  if Core.Core.ThemeManager.Theme.Name = 'Dark' then begin
    BFHighlighter.IoAttri.Foreground := clWhite;
    BFHighlighter.LoopAttri.Foreground := clLightRed;
    BFHighlighter.PointerAttri.Foreground := clLightBlue;
    BFHighlighter.MemoryAttri.Foreground := clLightGreen;
  end else begin
    BFHighlighter.IoAttri.Foreground := clBlack;
    BFHighlighter.LoopAttri.Foreground := clRed;
    BFHighlighter.PointerAttri.Foreground := clBlue;
    BFHighlighter.MemoryAttri.Foreground := clGreen;
  end;
end;

procedure TFormSourceCode.MemoSourceChange(Sender: TObject);
begin
  Core.Core.Project.Modified := True;
  Core.Core.CurrentTarget.Compiled := False;
  if Assigned(FOnUpdateInterface) then
    OnUpdateInterface(Self);
end;

procedure TFormSourceCode.MemoSourceKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
end;

procedure TFormSourceCode.MemoSourceKeyPress(Sender: TObject; var Key: char);
begin
end;

procedure TFormSourceCode.MemoSourceKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Assigned(FOnUpdateStatusBar) then
    FOnUpdateStatusBar(Self);
end;

procedure TFormSourceCode.MemoSourceMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(FOnUpdateStatusBar) then
    FOnUpdateStatusBar(Self);
end;

end.

