unit TargetJavascript;

interface

uses
  Classes, SysUtils, FileUtil, Target, BFTarget, Dialogs, LazFileUtils,
  Process, LCLIntf;

type

  { TTargetJavascript }

  TTargetJavascript = class(TBFTarget)
  private
    function GetMemoryCell: string;
  public
    WithHtml: Boolean;
    constructor Create; override;
    procedure Compile; override;
    procedure CompileHtml;
    procedure Run; override;
  end;


implementation

{ TTargetJavascript }

constructor TTargetJavascript.Create;
begin
  inherited;
  FName := 'Javascript';
  FSourceExtension := '.js';
  FImageIndex := 28;
  FCapabilities := [tcCompile, tcRun];
  CompilerPath := '';
  FCompiledExtension := '';
  FRunExtension := '';
end;

function TTargetJavascript.GetMemoryCell: string;
begin
  Result := 'Memory[Pos';
  if FProgram[FProgramIndex].RelIndex > 0 then
    Result := Result + ' + ' + IntToStr(FProgram[FProgramIndex].RelIndex)
  else if FProgram[FProgramIndex].RelIndex < 0 then
    Result := Result + ' - ' + IntToStr(Abs(FProgram[FProgramIndex].RelIndex));
  Result := Result + ']';
end;

procedure TTargetJavascript.Compile;
begin
  inherited;
  Indent := 0;
  FTargetCode := '';

  AddLine('var Memory = new Array(' + IntToStr(MemorySize) + ').fill(0);');
  AddLine('var Pos = 0;');
  AddLine('');
  FProgramIndex := 0;
  while FProgramIndex < FProgram.Count do begin
    case FProgram[FProgramIndex].Command of
      cmPointerInc: AddLine('Pos = Pos + ' + IntToStr(FProgram[FProgramIndex].Parameter) + ';');
      cmPointerDec: AddLine('Pos = Pos - ' + IntToStr(FProgram[FProgramIndex].Parameter) + ';');
      cmInc: AddLine(GetMemoryCell + ' = ' + GetMemoryCell + ' + ' + IntToStr(FProgram[FProgramIndex].Parameter) + ';');
      cmDec: AddLine(GetMemoryCell + ' = ' + GetMemoryCell + ' - ' + IntToStr(FProgram[FProgramIndex].Parameter) + ';');
      cmOutput: AddLine('document.getElementById("main").innerHTML += String.fromCharCode(' + GetMemoryCell + ');');
      cmInput: ; //AddLine(GetMemoryCell + ' = getchar();');
      cmSet: AddLine(GetMemoryCell + ' = ' + IntToStr(FProgram[FProgramIndex].Parameter) + ';');
      cmMultiply: AddLine(GetMemoryCell + ' = ' + GetMemoryCell + ' + Memory[Pos] * ' + IntToStr(FProgram[FProgramIndex].Parameter) + ';');
      cmLoopStart: begin
        AddLine('while(' + GetMemoryCell + ' != 0)');
        AddLine('{');
        Inc(Indent);
      end;
      cmLoopEnd: begin
        Dec(Indent);
        AddLine('}');
      end;
    end;
    Inc(FProgramIndex);
  end;
  CompileToFile;
  CompileHtml;
end;

procedure TTargetJavascript.CompileHtml;
var
  HtmlFile: string;
begin
  with TStringList.Create do
  try
    Text := '<!DOCTYPE html>' + LineEnding +
      '<html>' + LineEnding +
      '<body>' + LineEnding +
      '<pre id="main"></pre>' + LineEnding +
      '<script type="text/javascript" src="' + ExtractFileNameOnly(ProjectFileName) + SourceExtension + '"></script>' + LineEnding +
      '</body>' + LineEnding +
      '</html>';
    HtmlFile := ExtractFilePath(ProjectFileName) +
      CompiledDir + DirectorySeparator + Name + DirectorySeparator +
      ExtractFileNameOnly(ProjectFileName) + '.html';
    ForceDirectories(ExtractFilePath(HtmlFile));
    SaveToFile(HtmlFile);
  finally
    Free;
  end;
end;

procedure TTargetJavascript.Run;
var
  HtmlFile: string;
begin
  inherited;
  HtmlFile := ExtractFilePath(ProjectFileName) +
    CompiledDir + DirectorySeparator + Name + DirectorySeparator +
    ExtractFileNameOnly(ProjectFileName) + '.html';

  if FileExists(HtmlFile) then OpenURL(HtmlFile);
end;

end.

